@@ -1,13 +1,13 @@
 
 
-# Could make label attribute optional !!
+# Could make label attribute optional !
 descr <- function(X, Ndistinct = TRUE, higher = TRUE, table = TRUE,
                   Qprobs = c(0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99), cols = NULL,
                   label.attr = 'label', ...) {
-  nam <- deparse(substitute(X))
+  nam <- l1orlst(as.character(substitute(X)))
 
   armat <- function(x, y) c(x[1L], Ndist = y, x[-1L])
-  natrm <- function(x) if(is.na(names(x)[length(x)])) x[-length(x)] else x # Remove NA from table !!
+  natrm <- function(x) if(is.na(names(x)[length(x)])) x[-length(x)] else x # Remove NA from table !
 
   dotsok <- if(!missing(...)) names(substitute(c(...))[-1L]) %!in% c('pid','g') else TRUE
 
@@ -16,20 +16,20 @@ descr <- function(X, Ndistinct = TRUE, higher = TRUE, table = TRUE,
   descrnum <- if(is.numeric(Qprobs)) function(x, ...) list(Class = class(x), Label = attr(x, label.attr), Stats = numstats(x, ...),
                                                           Quant = quantile(x, probs = Qprobs, na.rm = TRUE)) else
                                          function(x, ...) list(Class = class(x), Label = attr(x, label.attr), Stats = numstats(x, ...))
-  # Could make this more efficient ???
+  # Could make this more efficient ?
   descrcat <- function(x, tab = table) if(tab) list(Class = class(x), Label = attr(x, label.attr),
                                                     Stats = if(Ndistinct) c(N = fNobsCpp(x), Ndist = fNdistinctCpp(x)) else `names<-`(fNobsCpp(x), 'Nobs'),
-                                                    Table = natrm(fNobs.default(x, x))) else # table(x). FNobs is a lot Faster, but includes NA as level !!
+                                                    Table = natrm(fNobs.default(x, x))) else # table(x). fNobs is a lot Faster, but includes NA as level !
                                                       list(Class = class(x), Label = attr(x, label.attr),
                                                            Stats = if(Ndistinct) c(N = fNobsCpp(x), Ndist = fNdistinctCpp(x)) else `names<-`(fNobsCpp(x), 'Nobs'))
   class(X) <- NULL
   if(!is.list(X)) X <- unclass(qDF(X))
   if(!is.null(cols)) X <- X[cols2int(cols, X, names(X))]
   res <- vector('list', length(X))
-  num <- vapply(X, is.numeric, TRUE, USE.NAMES = FALSE)
+  num <- vapply(unattrib(X), is.numeric, TRUE, USE.NAMES = FALSE)
   res[num] <- lapply(X[num], descrnum, ...)
   if(!all(num)) {
-    date <- vapply(X, is.Date, TRUE, USE.NAMES = FALSE)
+    date <- vapply(unattrib(X), is.Date, TRUE, USE.NAMES = FALSE)
     if(any(date)) {
       res[date] <- lapply(X[date], descrcat, FALSE)
       cat <- !(num | date)
@@ -50,7 +50,7 @@ print.descr <- function(x, n = 6, perc = TRUE, summary = TRUE, ...) {
   for(i in seq_along(x)) {
     xi <- x[[i]]
     namxi <- names(xi)
-    cat(nam[i]," (",xi[[1L]],"): ",xi[[2L]], "\n", sep = "")
+    cat(nam[i]," (",strclp(xi[[1L]]),"): ",xi[[2L]], "\n", sep = "")
     cat(namxi[3L], ": \n", sep = "")
     print.qsu(xi[[3L]])
     if(length(xi) > 3L) {
@@ -86,12 +86,13 @@ print.descr <- function(x, n = 6, perc = TRUE, summary = TRUE, ...) {
   }
   invisible(x)
 }
-
+# Not pasteclass !!
 
 # Note: This does not work for array stats (using g or pid.. )
 as.data.frame.descr <- function(x, ...) {
    if(attr(x, "arstat")) stop("Cannot handle arrays of statistics!")
-   r <- lapply(x, function(z) c(z[1:2], unlist(`names<-`(lapply(z[names(z) != "Table"][-(1:2)], as.vector, "list"), NULL), recursive = FALSE)))
+   r <- lapply(x, function(z) c(list(Class = strclp(z[[1L]]), Label = null2NA(z[[2L]])),
+        unlist(`names<-`(lapply(z[names(z) != "Table"][-(1:2)], as.vector, "list"), NULL), recursive = FALSE)))
    r <- .Call(C_rbindlist, r, TRUE, TRUE, "Variable")
    if(all(is.na(r[["Label"]]))) r[["Label"]] <- NULL
    attr(r, "row.names") <- .set_row_names(length(r[[1L]]))
